/*

 File: circle.cpp, 2003/06/17

 ACETAF, Version 3.0.0

 Copyright (C) Markus Neher, markus.neher@math.uni-karlsruhe.de
               Ingo Eble,    ingoeble@web.de

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include "circle.h"
#include "cinterval.h"

using Adapt::sup;

bool operator == (const Circle& A, const Circle& B)
{
  return ( A.getCenter() == B.getCenter() && A.getRadius() == B.getRadius() );
}

bool operator <  (const Circle& A, const Circle& B)
{
  if( A.getRadius() < B.getRadius() )
    {
      CInterval c1( A.getCenter() ), c2( B.getCenter() );

      double dc = sup( abs( c1 - c2 ) );//upper bound for |c1-c2|
      double dist;

#ifdef FILIB_VERSION
      ADD_UP( dist, dc , A.getRadius() );//upper bound for |c1-c2|+r1
#else //C-XSC Version
      dist = cxsc::_double( cxsc::addup( dc , A.getRadius() ) );//upper bound for |c1-c2|+r1
#endif

      return ( dist < B.getRadius() );// |c1-c2|+r1 < r2 
    }
  else return false;
}

bool operator <= (const Circle& A, const Circle& B)
{
  return ( A == B || A < B );
}

/*

  End of File: circle.cpp

*/

