/*

 File: gui_mainwindow.h, 2004/06/16

 ACETAF, Version 3.0.0

 Copyright (C) Markus Neher, markus.neher@math.uni-karlsruhe.de
               Ingo Eble,    ingoeble@web.de

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#ifndef _APPLICATION_H_INCLUDED
#define _APPLICATION_H_INCLUDED

#include <qmainwindow.h>
#include <qstring.h>

#include <vector>
#include <complex>

typedef std::complex<double> Complex;

#include "taylor.h"
#include "functions.h"

using Adapt::Interval;

//GUI
class QMultiLineEdit;
class QToolBar;
class QToolButton;
class QPopupMenu;

class InputWidget;
class OutputWindow;
class GarbageWindow;

class Calculation;
class ACheckControl;
class AHistory;
class MethodBase;
struct OptimResult;
struct APair;

//for the Output                                                                  
enum OutputFlag {  
  Head         = 1, 
  Head_Method1 = 2,
  Head_Method2 = 3,
  Head_Method3 = 4,
  Head_Method4 = 5,
  Body_Method1 = 6,   
  Body_Method2 = 7,   
  Body_Method3 = 8,   
  Body_Method4 = 9,   
  Tail         = 10  
};  

class MainWindow : public QMainWindow
{
  Q_OBJECT
    
 public:
  
  MainWindow();
  ~MainWindow();

 private slots:
    
  void loadFile();
  void saveFile();
  void saveFileAs();
  void printFile();
 
  void run(); 
  void stop();
  void acheck();
  void taylor();

  void description();
  void about();

  void calcFinished(const TaylorMemory<CInterval>*);
  void calcStopped();
  void acheckFinished(const APair*);
  void acheckStopped();
  void anErrorOccurred(const QString&);
  void getMessage(const QString&);
  void OutputActivated(const MethodBase*,const OptimResult*);
  void calcTaylorCoeffOnly(const Complex&,int);

 private:

  /*-----+
   | GUI |
   +-----*/

  QPrinter *printer;
  QToolBar *tools;
  QToolButton *fileOpen, *fileSave, *fileSaveAs, *filePrint, *runC, *stopC, *analyticityfunc, *tayloronlyfunc;
  QPopupMenu *file, *calculation;

  InputWidget *input;
  OutputWindow *output;
  GarbageWindow *garbage;

  QString filename;

  /*-------------+
   | Calculation |
   +-------------*/

  Calculation *calc;

  std::vector<int>      *bounds, *remainders;
  std::vector<double>   *omegare;
  std::vector<Interval> *omegaiv;

  ACheckControl *acc;
  AHistory *history;
  
  void setToolButtonEnabled(bool);
  void setMenuItemEnabled(bool);
  QString LineEditValue(int);
  QString outputString(OutputFlag,const OptimResult*,const TaylorMemory<CInterval>*); 
};

#endif

/*

  End of File: gui_mainwindow.h

*/
