/*

 File: mathexpr.cpp, 2004/06/14

 ACETAF, Version 3.0.0

 Copyright (C) Markus Neher, markus.neher@math.uni-karlsruhe.de
               Ingo Eble,    ingoeble@web.de

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include "mathexpr.h"
#include <iostream>

//Specialization of template prim<T> for T = int
//
#include <stdlib.h>

template<> prim<int>::prim(const std::string& s, const std::string& var) throw(syntax_error)
{
  if( !s.empty() )
    {
      std::string::const_iterator first = s.begin(),
                             last = s.end();
      OP = VZ = false;//default: OP = '*' and VZ = '+'
      fp1 = 0; fp2 = 0; fp3 = 0; expr = 0; return_value = 0;

      while( first != last )
	{
	  char ch( *first++ );
	  
	  switch( ch )
	    {
	    case PLUS: break;
	    case MINUS:
	      {
		VZ = !VZ;//because: (-1)*(-1) = +1 and (-1)*(-1)*(-1) = -1 and so on ...
		break;
	      }
	    case MUL: break;
	    case DIV:
	      {
		OP = true;
		break;
	      }
	    case LP:
	      {
		if( *(--last) != RP ) //the case: e.g. "(2*3+4)!" or "(2+1)" or "(4/(2+3))sin"
		  throw syntax_error("What does '" + s + " ' mean?");

		std::string content( first, last );
		expr = new MathExpr<int>( content, var );
		first = last;//leave the loop
		break;
	      }
	    case '0': case '1': case '2': case '3': case '4':
	    case '5': case '6': case '7': case '8': case '9':
	    case '.': 
	      {
		std::string number( --first, last );
		return_value = new int( atoi(number.c_str()) );
		if( OP && (*return_value) == 0 ) throw syntax_error("Division by zero.");
		first = last;//leave the loop
		break;
	      }
	    default:
	      {
		if( isalpha(ch) ) 
		  {
		    std::string name(""); name += ch;
		    while ( first != last && (ch = *first++) && isalnum(ch) ) 
		      {
			name += ch;
		      }
		    if( ch == LP ) 
		      { 
			throw syntax_error("No Function available for integer-evaluation.");
		      }
		    else 
		      { //No function! Look if it is the variable.
			if( name != var ) throw syntax_error("'" + name + "' unknown.");
			first = last;//leave the loop
		      }
		  }
		else 
		  {
		    throw syntax_error("Expression not finished.");
		    break;
		  }
	      }
	    }
	}
    }
  else
    {
      std::cerr<<"No string available."<<std::endl;
    }
}					 
//
//--------------------------------------------------------------------


//Specialization of evaluation operator for Taylor types
//
#include "taylor.h"

template<> CITaylor MathExpr<CITaylor>::operator () (const CITaylor& x)
{
  std::vector<term<CITaylor> >::iterator first = terms.begin(), last = terms.end();
  CITaylor sum( CInterval( 0.0 ), 0 , x.order() );
  sum.change_order( x.order() );

  while( first != last )
    {
      sum += (*first++)(x);
    }
  return sum;
}

template<> ITaylor MathExpr<ITaylor>::operator () (const ITaylor& x)
{
  std::vector<term<ITaylor> >::iterator first = terms.begin(), last = terms.end();
  ITaylor sum( Interval( 0.0 ), 0 , x.order() );
  sum.change_order( x.order() );

  while( first != last )
    {
      sum += (*first++)(x);
    }
  return sum;
}
//
//--------------------------------------------------------------------

//Specialization of template set_imag<T> for some types T 
//
template<> inline double* set_imag<double>() throw(syntax_error)
{
  throw syntax_error("real evaluation of complex expression is not possible.");
}
template<> inline Complex* set_imag<Complex>()
{
  return new Complex( 0, 1 );
}
template<> inline CInterval* set_imag<CInterval>()
{
  return new CInterval( Complex( 0, 1 ) );
}
template<> inline CITaylor* set_imag<CITaylor>()
{
  return new CITaylor( CInterval( Complex( 0, 1 ) ), 0 );
}
//
//--------------------------------------------------------------------

/*

  End of File: mathexpr.cpp

*/
