/*

 File: shareddata.h, 2003/06/17

 ACETAF, Version 3.0.0

 Copyright (C) Markus Neher, markus.neher@math.uni-karlsruhe.de
               Ingo Eble,    ingoeble@web.de

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#ifndef _SHAREDDATA_H_INCLUDED
#define _SHAREDDATA_H_INCLUDED

#include <qstring.h>
#include <qthread.h>

namespace Thread
{
  
  enum Status { 
    WORKING  = 0,
    ERROR    = 1,
    STOP     = 2,
    TIMEOVER = 3 
  };

}

class SharedData
{
  QString message;
  QString garbage;
  QString error;

  Thread::Status  thread;

  QMutex mutex;

 public:

  SharedData() : message(""), garbage(""), error(""), thread( Thread::WORKING ) {}
  ~SharedData() {}

  void appendToMessage(const QString& text)
    {
      mutex.lock();
      message += text;
      mutex.unlock();
    }
  void appendToGarbage(const QString& text)
    {
      mutex.lock();
      garbage += text;
      mutex.unlock();
    }
  void readMessage(QString& text)
    {
      mutex.lock();
      text = message;
      message = "";
      mutex.unlock();
    }
  void readGarbage(QString& text)
    {
      mutex.lock();
      text = garbage;
      garbage = "";
      mutex.unlock();
    }
  bool isThereGarbage() const
    {
      return !garbage.isEmpty();
    }
  bool isThereAMessage() const
    {
      return !message.isEmpty();
    }
  void setError(const QString& text)
    {
      mutex.lock();
      error = text;
      thread = Thread::ERROR;
      mutex.unlock();
    }
  void readError(QString& text)
    {
      mutex.lock();
      text = error;
      error = "";
      mutex.unlock();
    }
  void setStatus(const Thread::Status& stat)
    {
      mutex.lock();
      thread = stat;
      mutex.unlock();
    }
  const Thread::Status& readStatus() const
    {
      return thread;
    }
};

#endif

/*

  End of File: shareddata.h

*/
