/*

 File: taylor.cpp, 2004/06/14

 ACETAF, Version 3.0.0

 Copyright (C) Markus Neher, markus.neher@math.uni-karlsruhe.de
               Ingo Eble,    ingoeble@web.de

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include "taylor.h"

/*--------------------------------------+
 | Specializations of the operator:     |
 | bool FUNCTION<T>::operator () (...)  |
 +--------------------------------------*/
//For type CInterval
template<> bool LOG_DEF<CInterval>::operator () (const Taylor<CInterval>& x)
{
  return ( (x[0].re()).inf() <= 0.0 )&&( (x[0].im()).contains(0.0) );
}

template<> bool ASIN_DEF<CInterval>::operator () (const Taylor<CInterval>& x)
{
  return ( (((x[0].re()).inf() < -1.0) || ((x[0].re()).sup() > 1.0)) && (x[0].im()).contains(0.0) );
}

template<> bool ACOS_DEF<CInterval>::operator () (const Taylor<CInterval>& x)
{
  return ( (((x[0].re()).inf() < -1.0) || ((x[0].re()).sup() > 1.0)) && (x[0].im()).contains(0.0) );
}

template<> bool ASINH_DEF<CInterval>::operator () (const Taylor<CInterval>& x)//arsinh(z) = -i * arcsin(i*z)
{
  return ( (((x[0].im()).inf() < -1.0) || ((x[0].im()).sup() > 1.0)) && (x[0].re()).contains(0.0) );
}

template<> bool ACOSH_DEF<CInterval>::operator () (const Taylor<CInterval>& x)//arcosh(z) = i * arccos(z)
{
  return ( ((x[0].re()).inf() < 1.0 && (x[0].im()).contains(0.0)) || ((x[0].re()).contains(0.0)) );
}

template<> bool ATANH_DEF<CInterval>::operator () (const Taylor<CInterval>& x)
{
  return ( ((x[0].re()).contains(-1.0) || (x[0].re()).contains(1.0)) && (x[0].im()).contains(0.0) );
}

template<> bool ACOTH_DEF<CInterval>::operator () (const Taylor<CInterval>& x)
{
  return ( ((x[0].im()).contains(-1.0) || (x[0].im()).contains(1.0)) && (x[0].re()).contains(0.0) );
}

/*

  End of File: taylor.cpp

*/
